@@ -21,26 +21,27 @@
 #                 Byron Jones <bugzilla@glob.com.au>
 #                 Marc Schumann <wurblzap@gmail.com>
 
+package Bugzilla::CGI;
 use strict;
 
-package Bugzilla::CGI;
+use Bugzilla::Constants;
+use Bugzilla::Error;
+use Bugzilla::Util;
+
+use File::Basename;
 
 BEGIN {
-    if ($^O =~ /MSWin32/i) {
+    if (ON_WINDOWS) {
         # Help CGI find the correct temp directory as the default list
         # isn't Windows friendly (Bug 248988)
         $ENV{'TMPDIR'} = $ENV{'TEMP'} || $ENV{'TMP'} || "$ENV{'WINDIR'}\\TEMP";
     }
 }
 
-use CGI qw(-no_xhtml -oldstyle_urls :private_tempfiles :unique_headers SERVER_PUSH);
-
+use CGI qw(-no_xhtml -oldstyle_urls :private_tempfiles
+           :unique_headers SERVER_PUSH);
 use base qw(CGI);
 
-use Bugzilla::Constants;
-use Bugzilla::Error;
-use Bugzilla::Util;
-
 # We need to disable output buffering - see bug 179174
 $| = 1;
 
@@ -71,6 +72,12 @@ sub new {
     # Send appropriate charset
     $self->charset(Bugzilla->params->{'utf8'} ? 'UTF-8' : '');
 
+    # Redirect to urlbase/sslbase if we are not viewing an attachment.
+    my $script = basename($0);
+    if ($self->url_is_attachment_base and $script ne 'attachment.cgi') {
+        $self->redirect_to_urlbase();
+    }
+
     # Check for errors
     # All of the Bugzilla code wants to do this, so do it here instead of
     # in each script
@@ -103,12 +110,17 @@ sub new {
 sub canonicalise_query {
     my ($self, @exclude) = @_;
 
+    $self->convert_old_params();
     # Reconstruct the URL by concatenating the sorted param=value pairs
     my @parameters;
     foreach my $key (sort($self->param())) {
         # Leave this key out if it's in the exclude list
         next if lsearch(\@exclude, $key) != -1;
 
+        # Remove the Boolean Charts for standard query.cgi fields
+        # They are listed in the query URL already
+        next if $key =~ /^(field|type|value)(-\d+){3}$/;
+
         my $esc_key = url_quote($key);
 
         foreach my $value ($self->param($key)) {
@@ -123,9 +135,20 @@ sub canonicalise_query {
     return join("&", @parameters);
 }
 
+sub convert_old_params {
+    my $self = shift;
+
+    # bugidtype is now bug_id_type.
+    if ($self->param('bugidtype')) {
+        my $value = $self->param('bugidtype') eq 'exclude' ? 'nowords' : 'anyexact';
+        $self->param('bug_id_type', $value);
+        $self->delete('bugidtype');
+    }
+}
+
 sub clean_search_url {
     my $self = shift;
-    # Delete any empty URL parameter
+    # Delete any empty URL parameter.
     my @cgi_params = $self->param;
 
     foreach my $param (@cgi_params) {
@@ -142,10 +165,50 @@ sub clean_search_url {
         }
     }
 
-    # Delete certain parameters if the associated parameter is empty.
-    $self->delete('bugidtype')  if !$self->param('bug_id');
-    $self->delete('emailtype1') if !$self->param('email1');
-    $self->delete('emailtype2') if !$self->param('email2');
+    # Delete leftovers from the login form
+    $self->delete('Bugzilla_remember', 'GoAheadAndLogIn');
+
+    foreach my $num (1,2) {
+        # If there's no value in the email field, delete the related fields.
+        if (!$self->param("email$num")) {
+            foreach my $field qw(type assigned_to reporter qa_contact
+                                 cc longdesc) 
+            {
+                $self->delete("email$field$num");
+            }
+        }
+    }
+
+    # chfieldto is set to "Now" by default in query.cgi. But if none
+    # of the other chfield parameters are set, it's meaningless.
+    if (!defined $self->param('chfieldfrom') && !$self->param('chfield')
+        && !defined $self->param('chfieldvalue') && $self->param('chfieldto')
+        && lc($self->param('chfieldto')) eq 'now')
+    {
+        $self->delete('chfieldto');
+    }
+
+    # cmdtype "doit" is the default from query.cgi, but it's only meaningful
+    # if there's a remtype parameter.
+    if (defined $self->param('cmdtype') && $self->param('cmdtype') eq 'doit'
+        && !defined $self->param('remtype'))
+    {
+        $self->delete('cmdtype');
+    }
+
+    # "Reuse same sort as last time" is actually the default, so we don't
+    # need it in the URL.
+    if ($self->param('order') 
+        && $self->param('order') eq 'Reuse same sort as last time')
+    {
+        $self->delete('order');
+    }
+
+    # And now finally, if query_format is our only parameter, that
+    # really means we have no parameters, so we should delete query_format.
+    if ($self->param('query_format') && scalar($self->param()) == 1) {
+        $self->delete('query_format');
+    }
 }
 
 # Overwrite to ensure nph doesn't get set, and unset HEADERS_ONCE
@@ -221,17 +284,44 @@ sub header {
     return $self->SUPER::header(@_) || "";
 }
 
-# CGI.pm is not utf8-aware and passes data as bytes instead of UTF-8 strings.
 sub param {
     my $self = shift;
-    if (Bugzilla->params->{'utf8'} && scalar(@_) == 1) {
-        if (wantarray) {
-            return map { _fix_utf8($_) } $self->SUPER::param(@_);
+
+    # When we are just requesting the value of a parameter...
+    if (scalar(@_) == 1) {
+        my @result = $self->SUPER::param(@_); 
+
+        # Also look at the URL parameters, after we look at the POST 
+        # parameters. This is to allow things like login-form submissions
+        # with URL parameters in the form's "target" attribute.
+        if (!scalar(@result)
+            && $self->request_method && $self->request_method eq 'POST')
+        {
+            # Some servers fail to set the QUERY_STRING parameter, which
+            # causes undef issues
+            $ENV{'QUERY_STRING'} = '' unless exists $ENV{'QUERY_STRING'};
+            @result = $self->SUPER::url_param(@_);
         }
-        else {
-            return _fix_utf8(scalar $self->SUPER::param(@_));
+
+        # Fix UTF-8-ness of input parameters.
+        if (Bugzilla->params->{'utf8'}) {
+            @result = map { _fix_utf8($_) } @result;
         }
+
+        return wantarray ? @result : $result[0];
     }
+    # And for various other functions in CGI.pm, we need to correctly
+    # return the URL parameters in addition to the POST parameters when
+    # asked for the list of parameters.
+    elsif (!scalar(@_) && $self->request_method 
+           && $self->request_method eq 'POST') 
+    {
+        my @post_params = $self->SUPER::param;
+        my @url_params  = $self->url_param;
+        my %params = map { $_ => 1 } (@post_params, @url_params);
+        return keys %params;
+    }
+
     return $self->SUPER::param(@_);
 }
 
@@ -287,25 +377,89 @@ sub remove_cookie {
                        '-value'   => 'X');
 }
 
-# Redirect to https if required
-sub require_https {
-     my ($self, $url) = @_;
-     # Do not create query string if data submitted via XMLRPC
-     # since we want the data to be resubmitted over POST method.
-     my $query = Bugzilla->usage_mode == USAGE_MODE_WEBSERVICE ? 0 : 1;
-     # XMLRPC clients (SOAP::Lite at least) requires 301 to redirect properly
-     # and do not work with 302.
-     my $status = Bugzilla->usage_mode == USAGE_MODE_WEBSERVICE ? 301 : 302;
-     if (defined $url) {
-         $url .= $self->url('-path_info' => 1, '-query' => $query, '-relative' => 1);
-     } else {
-         $url = $self->self_url;
-         $url =~ s/^http:/https:/i;
-     }
-     print $self->redirect(-location => $url, -status => $status);
-     # When using XML-RPC with mod_perl, we need the headers sent immediately.
-     $self->r->rflush if $ENV{MOD_PERL};
-     exit;
+sub redirect_to_https {
+    my $self = shift;
+    my $sslbase = Bugzilla->params->{'sslbase'};
+    # If this is a POST, we don't want ?POSTDATA in the query string.
+    # We expect the client to re-POST, which may be a violation of
+    # the HTTP spec, but the only time we're expecting it often is
+    # in the WebService, and WebService clients usually handle this
+    # correctly.
+    $self->delete('POSTDATA');
+    my $url = $sslbase . $self->url('-path_info' => 1, '-query' => 1, 
+                                    '-relative' => 1);
+
+    # XML-RPC clients (SOAP::Lite at least) require a 301 to redirect properly
+    # and do not work with 302. Our redirect really is permanent anyhow, so
+    # it doesn't hurt to make it a 301.
+    print $self->redirect(-location => $url, -status => 301);
+
+    # When using XML-RPC with mod_perl, we need the headers sent immediately.
+    $self->r->rflush if $ENV{MOD_PERL};
+    exit;
+}
+
+# Redirect to the urlbase version of the current URL.
+sub redirect_to_urlbase {
+    my $self = shift;
+    my $path = $self->url('-path_info' => 1, '-query' => 1, '-relative' => 1);
+    print $self->redirect('-location' => correct_urlbase() . $path);
+    exit;
+}
+
+sub url_is_attachment_base {
+    my ($self, $id) = @_;
+    return 0 if !use_attachbase() or !i_am_cgi();
+    my $attach_base = Bugzilla->params->{'attachment_base'};
+    # If we're passed an id, we only want one specific attachment base
+    # for a particular bug. If we're not passed an ID, we just want to
+    # know if our current URL matches the attachment_base *pattern*.
+    my $regex;
+    if ($id) {
+        $attach_base =~ s/\%bugid\%/$id/;
+        $regex = quotemeta($attach_base);
+    }
+    else {
+        # In this circumstance we run quotemeta first because we need to
+        # insert an active regex meta-character afterward.
+        $regex = quotemeta($attach_base);
+        $regex =~ s/\\\%bugid\\\%/\\d+/;
+    }
+    $regex = "^$regex";
+    return ($self->self_url =~ $regex) ? 1 : 0;
+}
+
+##########################
+# Vars TIEHASH Interface #
+##########################
+
+# Fix the TIEHASH interface (scalar $cgi->Vars) to return and accept 
+# arrayrefs.
+sub STORE {
+    my $self = shift;
+    my ($param, $value) = @_;
+    if (defined $value and ref $value eq 'ARRAY') {
+        return $self->param(-name => $param, -value => $value);
+    }
+    return $self->SUPER::STORE(@_);
+}
+
+sub FETCH {
+    my ($self, $param) = @_;
+    return $self if $param eq 'CGI'; # CGI.pm did this, so we do too.
+    my @result = $self->param($param);
+    return undef if !scalar(@result);
+    return $result[0] if scalar(@result) == 1;
+    return \@result;
+}
+
+# For the Vars TIEHASH interface: the normal CGI.pm DELETE doesn't return 
+# the value deleted, but Perl's "delete" expects that value.
+sub DELETE {
+    my ($self, $param) = @_;
+    my $value = $self->FETCH($param);
+    $self->delete($param);
+    return $value;
 }
 
 1;
@@ -370,13 +524,17 @@ effectively removing the cookie.
 
 As its only argument, it takes the name of the cookie to expire.
 
-=item C<require_https($baseurl)>
+=item C<redirect_to_https>
 
-This routine redirects the client to a different location using the https protocol. 
-If the client is using XMLRPC, it will not retain the QUERY_STRING since XMLRPC uses POST.
+This routine redirects the client to the https version of the page that
+they're looking at, using the C<sslbase> parameter for the redirection.
 
-It takes an optional argument which will be used as the base URL.  If $baseurl
-is not provided, the current URL is used.
+Generally you should use L<Bugzilla::Util/do_ssl_redirect_if_required>
+instead of calling this directly.
+
+=item C<redirect_to_urlbase>
+
+Redirects from the current URL to one prefixed by the urlbase parameter.
 
 =back
 
