@@ -25,22 +25,29 @@ package Bugzilla::Install::Requirements;
 
 use strict;
 
-use Bugzilla::Install::Util qw(vers_cmp install_string);
+use Bugzilla::Constants;
+use Bugzilla::Install::Util qw(vers_cmp install_string 
+                               extension_requirement_packages);
 use List::Util qw(max);
 use Safe;
+use Term::ANSIColor;
 
 use base qw(Exporter);
 our @EXPORT = qw(
     REQUIRED_MODULES
     OPTIONAL_MODULES
+    FEATURE_FILES
 
     check_requirements
     check_graphviz
     have_vers
     install_command
+    map_files_to_features
 );
 
-use Bugzilla::Constants;
+# This is how many *'s are in the top of each "box" message printed
+# by checksetup.pl.
+use constant TABLE_WIDTH => 71;
 
 # The below two constants are subroutines so that they can implement
 # a hook. Other than that they are actually constants.
@@ -66,15 +73,30 @@ sub REQUIRED_MODULES {
         # CGI::Carp in 3.46 and 3.47 breaks Template Toolkit
         blacklist => ['^3\.46$', '^3\.47$'],
     },
+    {
+        package => 'Digest-SHA',
+        module  => 'Digest::SHA',
+        version => 0
+    },
     {
         package => 'TimeDate',
         module  => 'Date::Format',
         version => '2.21'
     },
+    # 0.28 fixed some important bugs in DateTime.
     {
-        package => 'PathTools',
-        module  => 'File::Spec',
-        version => '0.84'
+        package => 'DateTime',
+        module  => 'DateTime',
+        version => '0.28'
+    },
+    # 0.79 is required to work on Windows Vista and Windows Server 2008.
+    # As correctly detecting the flavor of Windows is not easy,
+    # we require this version for all Windows installations.
+    # 0.71 fixes a major bug affecting all platforms.
+    {
+        package => 'DateTime-TimeZone',
+        module  => 'DateTime::TimeZone',
+        version => ON_WINDOWS ? '0.79' : '0.71'
     },
     {
         package => 'DBI',
@@ -86,7 +108,7 @@ sub REQUIRED_MODULES {
     {
         package => 'Template-Toolkit',
         module  => 'Template',
-        version => ON_WINDOWS ? '2.22' : '2.15'
+        version => '2.22'
     },
     {
         package => 'Email-Send',
@@ -110,11 +132,16 @@ sub REQUIRED_MODULES {
         module  => 'Email::MIME::Modifier',
         version => '1.442'
     },
+    {
+        package => 'URI',
+        module  => 'URI',
+        version => 0
+    },
     );
 
-    my $all_modules = _get_extension_requirements(
-        'REQUIRED_MODULES', \@modules);
-    return $all_modules;
+    my $extra_modules = _get_extension_requirements('REQUIRED_MODULES');
+    push(@modules, @$extra_modules);
+    return \@modules;
 };
 
 sub OPTIONAL_MODULES {
@@ -123,13 +150,14 @@ sub OPTIONAL_MODULES {
         package => 'GD',
         module  => 'GD',
         version => '1.20',
-        feature => 'Graphical Reports, New Charts, Old Charts'
+        feature => [qw(graphical_reports new_charts old_charts)],
     },
     {
         package => 'Chart',
-        module  => 'Chart::Base',
-        version => '1.0',
-        feature => 'New Charts, Old Charts'
+        module  => 'Chart::Lines',
+        # Versions below 2.1 cannot be detected accurately.
+        version => '2.1',
+        feature => [qw(new_charts old_charts)],
     },
     {
         package => 'Template-GD',
@@ -137,89 +165,95 @@ sub OPTIONAL_MODULES {
         # on Template-Toolkits after 2.14, and still works with 2.14 and lower.
         module  => 'Template::Plugin::GD::Image',
         version => 0,
-        feature => 'Graphical Reports'
+        feature => ['graphical_reports'],
     },
     {
         package => 'GDTextUtil',
         module  => 'GD::Text',
         version => 0,
-        feature => 'Graphical Reports'
+        feature => ['graphical_reports'],
     },
     {
         package => 'GDGraph',
         module  => 'GD::Graph',
         version => 0,
-        feature => 'Graphical Reports'
+        feature => ['graphical_reports'],
     },
     {
         package => 'XML-Twig',
         module  => 'XML::Twig',
         version => 0,
-        feature => 'Move Bugs Between Installations'
+        feature => ['moving', 'updates'],
     },
     {
         package => 'MIME-tools',
         # MIME::Parser is packaged as MIME::Tools on ActiveState Perl
         module  => ON_WINDOWS ? 'MIME::Tools' : 'MIME::Parser',
         version => '5.406',
-        feature => 'Move Bugs Between Installations'
+        feature => ['moving'],
     },
     {
         package => 'libwww-perl',
         module  => 'LWP::UserAgent',
         version => 0,
-        feature => 'Automatic Update Notifications'
+        feature => ['updates'],
     },
     {
         package => 'PatchReader',
         module  => 'PatchReader',
         version => '0.9.4',
-        feature => 'Patch Viewer'
-    },
-    {
-        package => 'PerlMagick',
-        module  => 'Image::Magick',
-        version => 0,
-        feature => 'Optionally Convert BMP Attachments to PNGs'
+        feature => ['patch_viewer'],
     },
     {
         package => 'perl-ldap',
         module  => 'Net::LDAP',
         version => 0,
-        feature => 'LDAP Authentication'
+        feature => ['auth_ldap'],
     },
     {
         package => 'Authen-SASL',
         module  => 'Authen::SASL',
         version => 0,
-        feature => 'SMTP Authentication'
+        feature => ['smtp_auth'],
     },
     {
         package => 'RadiusPerl',
         module  => 'Authen::Radius',
         version => 0,
-        feature => 'RADIUS Authentication'
+        feature => ['auth_radius'],
     },
     {
         package => 'SOAP-Lite',
         module  => 'SOAP::Lite',
+        # 0.710.04 is required for correct UTF-8 handling, but .04 and .05 are
+        # affected by bug 468009.
+        version => '0.710.06',
+        feature => ['xmlrpc'],
+    },
+    {
+        package => 'JSON-RPC',
+        module  => 'JSON::RPC',
         version => 0,
-        # These versions (0.70 -> 0.710.05) are affected by bug 468009
-        blacklist => ['^0\.70', '^0\.710?\.0[1-5]$'],
-        feature => 'XML-RPC Interface'
+        feature => ['jsonrpc'],
+    },
+    {
+        package => 'Test-Taint',
+        module  => 'Test::Taint',
+        version => 0,
+        feature => ['jsonrpc', 'xmlrpc'],
     },
     {
         # We need the 'utf8_mode' method of HTML::Parser, for HTML::Scrubber.
         package => 'HTML-Parser',
         module  => 'HTML::Parser',
         version => '3.40',
-        feature => 'More HTML in Product/Group Descriptions'
+        feature => ['html_desc'],
     },
     {
         package => 'HTML-Scrubber',
         module  => 'HTML::Scrubber',
         version => 0,
-        feature => 'More HTML in Product/Group Descriptions'
+        feature => ['html_desc'],
     },
 
     # Inbound Email
@@ -227,13 +261,27 @@ sub OPTIONAL_MODULES {
         package => 'Email-MIME-Attachment-Stripper',
         module  => 'Email::MIME::Attachment::Stripper',
         version => 0,
-        feature => 'Inbound Email'
+        feature => ['inbound_email'],
     },
     {
         package => 'Email-Reply',
         module  => 'Email::Reply',
         version => 0,
-        feature => 'Inbound Email'
+        feature => ['inbound_email'],
+    },
+
+    # Mail Queueing
+    {
+        package => 'TheSchwartz',
+        module  => 'TheSchwartz',
+        version => 0,
+        feature => ['jobqueue'],
+    },
+    {
+        package => 'Daemon-Generic',
+        module  => 'Daemon::Generic',
+        version => 0,
+        feature => ['jobqueue'],
     },
 
     # mod_perl
@@ -241,40 +289,45 @@ sub OPTIONAL_MODULES {
         package => 'mod_perl',
         module  => 'mod_perl2',
         version => '1.999022',
-        feature => 'mod_perl'
+        feature => ['mod_perl'],
     },
     );
 
-    my $all_modules = _get_extension_requirements(
-        'OPTIONAL_MODULES', \@modules);
-    return $all_modules;
+    my $extra_modules = _get_extension_requirements('OPTIONAL_MODULES');
+    push(@modules, @$extra_modules);
+    return \@modules;
 };
 
-# This implements the install-requirements hook described in Bugzilla::Hook.
+# This maps features to the files that require that feature in order
+# to compile. It is used by t/001compile.t and mod_perl.pl.
+use constant FEATURE_FILES => (
+    jsonrpc       => ['Bugzilla/WebService/Server/JSONRPC.pm', 'jsonrpc.cgi'],
+    xmlrpc        => ['Bugzilla/WebService/Server/XMLRPC.pm', 'xmlrpc.cgi',
+                      'Bugzilla/WebService.pm', 'Bugzilla/WebService/*.pm'],
+    moving        => ['importxml.pl'],
+    auth_ldap     => ['Bugzilla/Auth/Verify/LDAP.pm'],
+    auth_radius   => ['Bugzilla/Auth/Verify/RADIUS.pm'],
+    inbound_email => ['email_in.pl'],
+    jobqueue      => ['Bugzilla/Job/*', 'Bugzilla/JobQueue.pm',
+                      'Bugzilla/JobQueue/*', 'jobqueue.pl'],
+    patch_viewer  => ['Bugzilla/Attachment/PatchReader.pm'],
+    updates       => ['Bugzilla/Update.pm'],
+);
+
+# This implements the REQUIRED_MODULES and OPTIONAL_MODULES stuff
+# described in in Bugzilla::Extension.
 sub _get_extension_requirements {
-    my ($function, $base_modules) = @_;
-    my @all_modules;
-    # get a list of all extensions
-    my @extensions = glob(bz_locations()->{'extensionsdir'} . "/*");
-    foreach my $extension (@extensions) {
-        my $file = "$extension/code/install-requirements.pl";
-        if (-e $file) {
-            my $safe = new Safe;
-            # This is a very liberal Safe.
-            $safe->permit(qw(:browse require entereval caller));
-            $safe->rdo($file);
-            if ($@) {
-                warn $@;
-                next;
-            }
-            my $modules = eval { &{$safe->varglob($function)}($base_modules) };
-            next unless $modules;
-            push(@all_modules, @$modules);
+    my ($function) = @_;
+
+    my $packages = extension_requirement_packages();
+    my @modules;
+    foreach my $package (@$packages) {
+        if ($package->can($function)) {
+            my $extra_modules = $package->$function;
+            push(@modules, @$extra_modules);
         }
     }
-
-    unshift(@all_modules, @$base_modules);
-    return \@all_modules;
+    return \@modules;
 };
 
 sub check_requirements {
@@ -337,148 +390,110 @@ sub _get_activestate_build_id {
 sub print_module_instructions {
     my ($check_results, $output) = @_;
 
-    # We only print these notes if we have to.
-    if ((!$output && @{$check_results->{missing}})
-        || ($output && $check_results->{any_missing}))
-    {
-        
-        if (ON_WINDOWS) {
+    # First we print the long explanatory messages.
 
-            print "\n* NOTE: You must run any commands listed below as "
-                  . ROOT_USER . ".\n\n";
-
-            my $perl_ver = sprintf('%vd', $^V);
-            
-            # URL when running Perl 5.8.x.
-            my $url_to_theory58S = 'http://theoryx5.uwinnipeg.ca/ppms';
-            my $repo_up_cmd =
-'*                                                                     *';
-            # Packages for Perl 5.10 are not compatible with Perl 5.8.
-            if (vers_cmp($perl_ver, '5.10') > -1) {
-                $url_to_theory58S = 'http://cpan.uwinnipeg.ca/PPMPackages/10xx/';
-            }
-            # ActivePerl older than revision 819 require an additional command.
-            if (_get_activestate_build_id() < 819) {
-                $repo_up_cmd = <<EOT;
-*                                                                     *
-* Then you have to do (also as an Administrator):                     *
-*                                                                     *
-*   ppm repo up theory58S                                             *
-*                                                                     *
-* Do that last command over and over until you see "theory58S" at the *
-* top of the displayed list.                                          *
-EOT
-            }
-            print <<EOT;
-***********************************************************************
-* Note For Windows Users                                              *
-***********************************************************************
-* In order to install the modules listed below, you first have to run * 
-* the following command as an Administrator:                          *
-*                                                                     *
-*   ppm repo add theory58S $url_to_theory58S
-$repo_up_cmd
-***********************************************************************
-EOT
-        }
-    }
-
-    # Required Modules
-    if (my @missing = @{$check_results->{missing}}) {
-        print <<EOT;
-***********************************************************************
-* REQUIRED MODULES                                                    *
-***********************************************************************
-* Bugzilla requires you to install some Perl modules which are either *
-* missing from your system, or the version on your system is too old. *
-*                                                                     *
-* The latest versions of each module can be installed by running the  *
-* commands below.                                                     *
-***********************************************************************
-EOT
-
-        print "COMMANDS:\n\n";
-        foreach my $package (@missing) {
-            my $command = install_command($package);
-            print "    $command\n";
-        }
-        print "\n";
+    if (scalar @{$check_results->{missing}}) {
+        print install_string('modules_message_required');
     }
 
     if (!$check_results->{one_dbd}) {
-        print <<EOT;
-***********************************************************************
-* DATABASE ACCESS                                                     *
-***********************************************************************
-* In order to access your database, Bugzilla requires that the        *
-* correct "DBD" module be installed for the database that you are     *
-* running.                                                            *
-*                                                                     *
-* Pick and run the correct command below for the database that you    *
-* plan to use with Bugzilla.                                          *
-***********************************************************************
-COMMANDS:
-
-EOT
-
-        my %db_modules = %{DB_MODULE()};
-        foreach my $db (keys %db_modules) {
-            my $command = install_command($db_modules{$db}->{dbd});
-            printf "%10s: \%s\n", $db_modules{$db}->{name}, $command;
-            print ' ' x 12 . "Minimum version required: "
-                  . $db_modules{$db}->{dbd}->{version} . "\n";
-        }
-        print "\n";
+        print install_string('modules_message_db');
     }
 
-    return unless $output;
-
-    if (my @missing = @{$check_results->{optional}}) {
-        print <<EOT;
-**********************************************************************
-* OPTIONAL MODULES                                                   *
-**********************************************************************
-* Certain Perl modules are not required by Bugzilla, but by          *
-* installing the latest version you gain access to additional        *
-* features.                                                          *
-*                                                                    *
-* The optional modules you do not have installed are listed below,   *
-* with the name of the feature they enable. If you want to install   *
-* one of these modules, just run the appropriate command in the      *
-* "COMMANDS TO INSTALL" section.                                     *
-**********************************************************************
-
-EOT
+    if (my @missing = @{$check_results->{optional}} and $output) {
+        print install_string('modules_message_optional');
         # Now we have to determine how large the table cols will be.
         my $longest_name = max(map(length($_->{package}), @missing));
 
         # The first column header is at least 11 characters long.
         $longest_name = 11 if $longest_name < 11;
 
-        # The table is 71 characters long. There are seven mandatory
+        # The table is TABLE_WIDTH characters long. There are seven mandatory
         # characters (* and space) in the string. So, we have a total
-        # of 64 characters to work with.
-        my $remaining_space = 64 - $longest_name;
-        print '*' x 71 . "\n";
+        # of TABLE_WIDTH - 7 characters to work with.
+        my $remaining_space = (TABLE_WIDTH - 7) - $longest_name;
+        print '*' x TABLE_WIDTH . "\n";
         printf "* \%${longest_name}s * %-${remaining_space}s *\n",
                'MODULE NAME', 'ENABLES FEATURE(S)';
-        print '*' x 71 . "\n";
+        print '*' x TABLE_WIDTH . "\n";
         foreach my $package (@missing) {
             printf "* \%${longest_name}s * %-${remaining_space}s *\n",
-                   $package->{package}, $package->{feature};
+                   $package->{package}, 
+                   _translate_feature($package->{feature});
         }
-        print '*' x 71 . "\n";
+    }
 
-        print "COMMANDS TO INSTALL:\n\n";
+    # We only print the PPM repository note if we have to.
+    if ((!$output && @{$check_results->{missing}})
+        || ($output && $check_results->{any_missing}))
+    {
+        if (ON_WINDOWS) {
+            my $perl_ver = sprintf('%vd', $^V);
+            
+            # URL when running Perl 5.8.x.
+            my $url_to_theory58S = 'http://theoryx5.uwinnipeg.ca/ppms';
+            # Packages for Perl 5.10 are not compatible with Perl 5.8.
+            if (vers_cmp($perl_ver, '5.10') > -1) {
+                $url_to_theory58S = 'http://cpan.uwinnipeg.ca/PPMPackages/10xx/';
+            }
+            print colored(install_string('ppm_repo_add', 
+                                 { theory_url => $url_to_theory58S }), 'red');
+            # ActivePerls older than revision 819 require an additional command.
+            if (_get_activestate_build_id() < 819) {
+                print install_string('ppm_repo_up');
+            }
+        }
+
+        # If any output was required, we want to close the "table"
+        print "*" x TABLE_WIDTH . "\n";
+    }
+
+    # And now we print the actual installation commands.
+
+    if (my @missing = @{$check_results->{optional}} and $output) {
+        print install_string('commands_optional') . "\n\n";
         foreach my $module (@missing) {
             my $command = install_command($module);
             printf "%15s: $command\n", $module->{package};
         }
+        print "\n";
     }
 
-    if ($output && $check_results->{any_missing} && !ON_WINDOWS) {
+    if (!$check_results->{one_dbd}) {
+        print install_string('commands_dbd') . "\n";
+        my %db_modules = %{DB_MODULE()};
+        foreach my $db (keys %db_modules) {
+            my $command = install_command($db_modules{$db}->{dbd});
+            printf "%10s: \%s\n", $db_modules{$db}->{name}, $command;
+        }
+        print "\n";
+    }
+
+    if (my @missing = @{$check_results->{missing}}) {
+        print colored(install_string('commands_required'), 'red') . "\n";
+        foreach my $package (@missing) {
+            my $command = install_command($package);
+            print "    $command\n";
+        }
+    }
+
+    if ($output && $check_results->{any_missing} && !ON_WINDOWS
+        && !$check_results->{hide_all}) 
+    {
         print install_string('install_all', { perl => $^X });
     }
+    if (!$check_results->{pass}) {
+        print colored(install_string('installation_failed'), 'red') . "\n\n";
+    }
+}
+
+sub _translate_feature {
+    my $features = shift;
+    my @strings;
+    foreach my $feature (@$features) {
+        push(@strings, install_string("feature_$feature"));
+    }
+    return join(', ', @strings);
 }
 
 sub check_graphviz {
@@ -525,8 +540,9 @@ sub have_vers {
 
     eval "require $module;";
 
-    # VERSION is provided by UNIVERSAL::
-    my $vnum = eval { $module->VERSION } || -1;
+    # VERSION is provided by UNIVERSAL::, and can be called even if
+    # the module isn't loaded.
+    my $vnum = $module->VERSION || -1;
 
     # CGI's versioning scheme went 2.75, 2.751, 2.752, 2.753, 2.76
     # That breaks the standard version tests, so we need to manually correct
@@ -534,6 +550,10 @@ sub have_vers {
     if ($module eq 'CGI' && $vnum =~ /(2\.7\d)(\d+)/) {
         $vnum = $1 . "." . $2;
     }
+    # CPAN did a similar thing, where it has versions like 1.9304.
+    if ($module eq 'CPAN' and $vnum =~ /^(\d\.\d{2})\d{2}$/) {
+        $vnum = $1;
+    }
 
     my $vstr;
     if ($vnum eq "-1") { # string compare just in case it's non-numeric
@@ -559,8 +579,9 @@ sub have_vers {
         my $want_string  = $wanted ? "v$wanted" : install_string('any');
 
         $ok = "$ok:" if $ok;
-        printf "%s %19s %-9s $ok $vstr $black_string\n",
-            install_string('checking_for'), $package, "($want_string)";
+        my $str = sprintf "%s %19s %-9s $ok $vstr $black_string\n",
+                    install_string('checking_for'), $package, "($want_string)";
+        print $vok ? $str : colored($str, 'red');
     }
     
     return $vok ? 1 : 0;
@@ -583,6 +604,21 @@ sub install_command {
     return sprintf $command, $package;
 }
 
+# This does a reverse mapping for FEATURE_FILES.
+sub map_files_to_features {
+    my %features = FEATURE_FILES;
+    my %files;
+    foreach my $feature (keys %features) {
+        my @my_files = @{ $features{$feature} };
+        foreach my $pattern (@my_files) {
+            foreach my $file (glob $pattern) {
+                $files{$file} = $feature;
+            }
+        }
+    }
+    return \%files;
+}
+
 1;
 
 __END__
@@ -600,16 +636,42 @@ perl modules it requires.)
 
 =head1 CONSTANTS
 
-=over 4
+=over
 
 =item C<REQUIRED_MODULES>
 
 An arrayref of hashrefs that describes the perl modules required by 
-Bugzilla. The hashes have two keys, C<name> and C<version>, which
-represent the name of the module and the version that we require.
+Bugzilla. The hashes have three keys: 
+
+=over
+
+=item C<package> - The name of the Perl package that you'd find on
+CPAN for this requirement. 
+
+=item C<module> - The name of a module that can be passed to the
+C<install> command in C<CPAN.pm> to install this module.
+
+=item C<version> - The version of this module that we require, or C<0>
+if any version is acceptable.
 
 =back
 
+=item C<OPTIONAL_MODULES>
+
+An arrayref of hashrefs that describes the perl modules that add
+additional features to Bugzilla if installed. Its hashes have all
+the fields of L</REQUIRED_MODULES>, plus a C<feature> item--an arrayref
+of strings that describe what features require this module.
+
+=item C<FEATURE_FILES>
+
+A hashref that describes what files should only be compiled if a certain
+feature is enabled. The feature is the key, and the values are arrayrefs
+of file names (which are passed to C<glob>, so shell patterns work).
+
+=back
+
+
 =head1 SUBROUTINES
 
 =over 4
@@ -692,4 +754,9 @@ Returns:     C<1> if the check was successful, C<0> otherwise.
 
  Returns:     nothing
 
+=item C<map_files_to_features>
+
+Returns a hashref where file names are the keys and the value is the feature
+that must be enabled in order to compile that file.
+
 =back
