@@ -87,15 +87,10 @@ use Bugzilla::Status;
 
 use MIME::Base64;
 use MIME::Parser;
-use Date::Format;
 use Getopt::Long;
 use Pod::Usage;
 use XML::Twig;
 
-# We want to capture errors and handle them here rather than have the Template
-# code barf all over the place.
-Bugzilla->usage_mode(Bugzilla::Constants::USAGE_MODE_CMDLINE);
-
 my $debug = 0;
 my $mail  = '';
 my $attach_path = '';
@@ -517,7 +512,7 @@ sub process_bug {
         $long_desc{'isprivate'} = $comment->{'att'}->{'isprivate'} || 0;
 
         # if one of the comments is private we need to set this flag
-        if ( $long_desc{'isprivate'} && $exporter->in_group($params->{'insidergroup'})) {
+        if ( $long_desc{'isprivate'} && $exporter->is_insider) {
             $private = 1;
         }
         my $data = $comment->field('thetext');
@@ -528,6 +523,8 @@ sub process_bug {
             $data = decode_base64($data);
         }
 
+        # For backwards-compatibility with Bugzillas before 3.6:
+        #
         # If we leave the attachment ID in the comment it will be made a link
         # to the wrong attachment. Since the new attachment ID is unknown yet
         # let's strip it out for now. We will make a comment with the right ID
@@ -574,8 +571,7 @@ sub process_bug {
     my $comments;
 
     $comments .= "\n\n--- Bug imported by $exporter_login ";
-    $comments .= time2str( "%Y-%m-%d %H:%M", time ) . " ";
-    $comments .= $params->{'timezone'};
+    $comments .= format_time(scalar localtime(time()), '%Y-%m-%d %R %Z') . " ";
     $comments .= " ---\n\n";
     $comments .= "This bug was previously known as _bug_ $bug_fields{'bug_id'} at ";
     $comments .= $urlbase . "show_bug.cgi?id=" . $bug_fields{'bug_id'} . "\n";
@@ -808,8 +804,7 @@ sub process_bug {
 
     # Process time fields
     if ( $params->{"timetrackinggroup"} ) {
-        my $date = format_time( $bug_fields{'deadline'}, "%Y-%m-%d" )
-          || undef;
+        my $date = validate_date( $bug_fields{'deadline'} ) ? $bug_fields{'deadline'} : undef;
         push( @values, $date );
         push( @query,  "deadline" );
         if ( defined $bug_fields{'estimated_time'} ) {
@@ -918,7 +913,7 @@ sub process_bug {
     
     # Check everconfirmed 
     my $everconfirmed;
-    if ($product->votes_to_confirm) {
+    if ($product->allows_unconfirmed) {
         $everconfirmed = $bug_fields{'everconfirmed'} || 0;
     }
     else {
@@ -1205,7 +1200,7 @@ sub process_bug {
             $err .= "No attachment ID specified, dropping attachment\n";
             next;
         }
-        if (!$exporter->in_group($params->{'insidergroup'}) && $att->{'isprivate'}){
+        if (!$exporter->is_insider && $att->{'isprivate'}) {
             $err .= "Exporter not in insidergroup and attachment marked private.\n";
             $err .= "   Marking attachment public\n";
             $att->{'isprivate'} = 0;
@@ -1258,7 +1253,7 @@ sub process_bug {
 
     # Insert longdesc and append any errors
     my $worktime = $bug_fields{'actual_time'} || 0.0;
-    $worktime = 0.0 if (!$exporter->in_group($params->{'timetrackinggroup'}));
+    $worktime = 0.0 if (!$exporter->is_timetracker);
     $long_description .= "\n" . $comments;
     if ($err) {
         $long_description .= "\n$err\n";
