@@ -26,7 +26,7 @@ use warnings;
 # CPAN has chdir'ed around. We do all of this in this funny order to
 # make sure that we use the lib/ modules instead of the base Perl modules,
 # in case the lib/ modules are newer.
-use Cwd qw(abs_path);
+use Cwd qw(abs_path cwd);
 use lib abs_path('.');
 use Bugzilla::Constants;
 use lib abs_path(bz_locations()->{ext_libpath});
@@ -35,15 +35,19 @@ use Bugzilla::Install::CPAN;
 
 use Bugzilla::Constants;
 use Bugzilla::Install::Requirements;
+use Bugzilla::Install::Util qw(bin_loc init_console vers_cmp);
 
 use Data::Dumper;
 use Getopt::Long;
 use Pod::Usage;
 
-our %switch;
+init_console();
 
+my @original_args = @ARGV;
+my $original_dir = cwd();
+our %switch;
 GetOptions(\%switch, 'all|a', 'upgrade-all|u', 'show-config|s', 'global|g',
-                     'help|h');
+                     'shell', 'help|h');
 
 pod2usage({ -verbose => 1 }) if $switch{'help'};
 
@@ -58,15 +62,15 @@ pod2usage({ -verbose => 0 }) if (!%switch && !@ARGV);
 set_cpan_config($switch{'global'});
 
 if ($switch{'show-config'}) {
-  print Dumper($CPAN::Config);
-  exit;
+    print Dumper($CPAN::Config);
+    exit;
 }
 
-my $can_notest = 1;
-if (substr(CPAN->VERSION, 0, 3) < 1.8) {
-    $can_notest = 0;
-    print "* Note: If you upgrade your CPAN module, installs will be faster.\n";
-    print "* You can upgrade CPAN by doing: $^X install-module.pl CPAN\n";
+check_cpan_requirements($original_dir, \@original_args);
+
+if ($switch{'shell'}) {
+    CPAN::shell();
+    exit;
 }
 
 if ($switch{'all'} || $switch{'upgrade-all'}) {
@@ -93,12 +97,13 @@ if ($switch{'all'} || $switch{'upgrade-all'}) {
         # configuration, and really should be installed on its own.
         next if $cpan_name eq 'mod_perl2';
         next if $cpan_name eq 'DBD::Oracle' and !$ENV{ORACLE_HOME};
-        install_module($cpan_name, $can_notest);
+        next if $cpan_name eq 'DBD::Pg' and !bin_loc('pg_config');
+        install_module($cpan_name);
     }
 }
 
 foreach my $module (@ARGV) {
-    install_module($module, $can_notest);
+    install_module($module);
 }
 
 __END__
@@ -114,6 +119,7 @@ This script does not run on Windows.
   ./install-module.pl --all [--global]
   ./install-module.pl --upgrade-all [--global]
   ./install-module.pl --show-config
+  ./install-module.pl --shell
 
   Do "./install-module.pl --help" for more information.
 
@@ -154,6 +160,10 @@ have them installed.
 
 Prints out the CPAN configuration in raw Perl format. Useful for debugging.
 
+=item B<--shell>
+
+Starts a CPAN shell using the configuration of F<install-module.pl>.
+
 =item B<--help>
 
 Shows this help.
