@@ -60,10 +60,7 @@
 'search/boolean-charts.html.tmpl' => [
   '"field${chartnum}-${rownum}-${colnum}"', 
   '"value${chartnum}-${rownum}-${colnum}"', 
-  '"type${chartnum}-${rownum}-${colnum}"', 
   'field.name', 
-  'type.name',
-  'type.description', 
   '"${chartnum}-${rownum}-${newor}"', 
   '"${chartnum}-${newand}-0"', 
   'newchart',
@@ -71,7 +68,6 @@
 ],
 
 'search/form.html.tmpl' => [
-  'qv.value',
   'qv.name',
   'qv.description',
   'field.name',
@@ -96,24 +92,6 @@
   'request.attach_id', 
 ],
 
-'reports/components.html.tmpl' => [
-  'numcols',
-],
-
-'reports/duplicates-table.html.tmpl' => [
-  'column.name', 
-  'column.description',
-  'bug.count', 
-  'bug.delta', 
-],
-
-'reports/duplicates.html.tmpl' => [
-  'bug_ids_string', 
-  'maxrows',
-  'changedsince',
-  'reverse',
-],
-
 'reports/keywords.html.tmpl' => [
   'keyword.bug_count', 
 ],
@@ -147,13 +125,6 @@
   'cumulate',
 ],
 
-'reports/duplicates.rdf.tmpl' => [
-  'template_version', 
-  'bug.id', 
-  'bug.count', 
-  'bug.delta', 
-],
-
 'reports/chart.html.tmpl' => [
   'width', 
   'height', 
@@ -184,10 +155,6 @@
   'default.series_id', 
 ],
 
-'list/change-columns.html.tmpl' => [
-  'column', 
-],
-
 'list/edit-multiple.html.tmpl' => [
   'group.id', 
   'menuname',
@@ -265,10 +232,6 @@
 ],
 
 'global/site-navigation.html.tmpl' => [
-  'bug_list.first', 
-  'bug_list.$prev_bug', 
-  'bug_list.$next_bug', 
-  'bug_list.last', 
   'bug.bug_id', 
   'bug.votes', 
 ],
@@ -312,17 +275,9 @@
   '" colspan=\"$colspan\"" IF colspan',
   '" size=\"$size\"" IF size',
   '" maxlength=\"$maxlength\"" IF maxlength',
-  'flag.status',
   '" spellcheck=\"$spellcheck\"" IF spellcheck',
 ],
 
-'bug/navigate.html.tmpl' => [
-  'bug_list.first', 
-  'bug_list.last', 
-  'bug_list.$prev_bug', 
-  'bug_list.$next_bug', 
-],
-
 'bug/show-multiple.html.tmpl' => [
   'attachment.id', 
   'flag.status',
@@ -343,6 +298,10 @@
   'subtotal FILTER format("%.2f")',
   'work_time FILTER format("%.2f")',
   'global.total FILTER format("%.2f")',
+  'global.remaining FILTER format("%.2f")',
+  'global.estimated FILTER format("%.2f")',
+  'bugs.$id.remaining_time FILTER format("%.2f")',
+  'bugs.$id.estimated_time FILTER format("%.2f")',
 ],
 
 
@@ -387,7 +346,6 @@
 
 'bug/activity/table.html.tmpl' => [
   'change.attachid', 
-  'change.field', 
 ],
 
 'attachment/create.html.tmpl' => [
@@ -403,7 +361,8 @@
 'attachment/edit.html.tmpl' => [
   'attachment.id', 
   'attachment.bug_id', 
-  'a', 
+  'a',
+  'editable_or_hide',
 ],
 
 'attachment/list.html.tmpl' => [
@@ -432,8 +391,6 @@
   'bugid',
   'oldid',
   'newid',
-  'style',
-  'javascript',
   'patch.id',
 ],
 
@@ -446,8 +403,6 @@
   'section_num',
   'current_line_old',
   'current_line_new',
-  'curr_old',
-  'curr_new'
 ],
 
 'admin/admin.html.tmpl' => [
@@ -458,6 +413,11 @@
   'link_uri'
 ],
 
+'admin/custom_fields/cf-js.js.tmpl' => [
+  'constants.FIELD_TYPE_SINGLE_SELECT',
+  'constants.FIELD_TYPE_MULTI_SELECT',
+],
+
 'admin/params/common.html.tmpl' => [
   'sortlist_separator', 
 ],
@@ -467,22 +427,17 @@
 ],
 
 'admin/products/groupcontrol/edit.html.tmpl' => [
-  'group.bugcount', 
-  'group.id', 
-  'const.CONTROLMAPNA', 
-  'const.CONTROLMAPSHOWN', 
-  'const.CONTROLMAPDEFAULT', 
-  'const.CONTROLMAPMANDATORY', 
+  'group.id',
+  'constants.CONTROLMAPNA', 
+  'constants.CONTROLMAPSHOWN',
+  'constants.CONTROLMAPDEFAULT',
+  'constants.CONTROLMAPMANDATORY',
 ],
 
 'admin/products/list.html.tmpl' => [
   'classification_url_part', 
 ],
 
-'admin/products/confirm-delete.html.tmpl' => [
-  'classification_url_part', 
-],
-
 'admin/products/footer.html.tmpl' => [
   'classification_url_part', 
   'classification_text', 
@@ -554,8 +509,8 @@
   'new_status.id',
 ],
 
-'account/login.html.tmpl' => [
-  'target', 
+'account/auth/login-small.html.tmpl' => [
+  'qs_suffix',
 ],
 
 'account/prefs/email.html.tmpl' => [
