@@ -34,6 +34,7 @@ use Bugzilla::Search;
 use Bugzilla::User;
 use Bugzilla::Mailer;
 use Bugzilla::Util;
+use Bugzilla::Group;
 
 # create some handles that we'll need
 my $template = Bugzilla->template;
@@ -64,7 +65,8 @@ my $sth_next_scheduled_event = $dbh->prepare(
     " whine_schedules.eventid, " .
     " whine_events.owner_userid, " .
     " whine_events.subject, " .
-    " whine_events.body " .
+    " whine_events.body, " .
+    " whine_events.mailifnobugs " .
     "FROM whine_schedules " .
     "LEFT JOIN whine_events " .
     " ON whine_events.id = whine_schedules.eventid " .
@@ -199,6 +201,7 @@ $sched_h->finish();
 #   users   - array of user objects for recipients
 #   subject - Subject line for the email
 #   body    - the text inserted above the bug lists
+#   mailifnobugs - send message even if there are no query or query results
 
 sub get_next_event {
     my $event = {};
@@ -213,7 +216,7 @@ sub get_next_event {
         my $fetched = $sth_next_scheduled_event->fetch;
         $sth_next_scheduled_event->finish;
         return undef unless $fetched;
-        my ($eventid, $owner_id, $subject, $body) = @{$fetched};
+        my ($eventid, $owner_id, $subject, $body, $mailifnobugs) = @{$fetched};
 
         my $owner = Bugzilla::User->new($owner_id);
 
@@ -250,7 +253,7 @@ sub get_next_event {
                         $groupname, $owner);
                     if ($group_id) {
                         my $glist = join(',',
-                            @{Bugzilla::User->flatten_group_membership(
+                            @{Bugzilla::Group->flatten_group_membership(
                             $group_id)});
                         $sth = $dbh->prepare("SELECT user_id FROM " .
                                              "user_group_map " .
@@ -281,6 +284,7 @@ sub get_next_event {
                     'mailto'  => \@users,
                     'subject' => $subject,
                     'body'    => $body,
+                    'mailifnobugs' => $mailifnobugs,
             };
         }
     }
@@ -295,6 +299,7 @@ sub get_next_event {
 #   mailto  (array of user objects for mail targets)
 #   subject (subject line for message)
 #   body    (text blurb at top of message)
+#   mailifnobugs (send message even if there are no query or query results)
 while (my $event = get_next_event) {
 
     my $eventid = $event->{'eventid'};
@@ -315,12 +320,14 @@ while (my $event = get_next_event) {
         # run the queries for this schedule
         my $queries = run_queries($args);
 
-        # check to make sure there is something to output
-        my $there_are_bugs = 0;
-        for my $query (@{$queries}) {
-            $there_are_bugs = 1 if scalar @{$query->{'bugs'}};
+        # If mailifnobugs is false, make sure there is something to output
+        if (!$event->{'mailifnobugs'}) {
+            my $there_are_bugs = 0;
+            for my $query (@{$queries}) {
+                $there_are_bugs = 1 if scalar @{$query->{'bugs'}};
+            }
+            next unless $there_are_bugs;
         }
-        next unless $there_are_bugs;
 
         $args->{'queries'} = $queries;
 
@@ -424,16 +431,15 @@ sub run_queries {
         next unless $savedquery;    # silently ignore missing queries
 
         # Execute the saved query
-        my @searchfields = (
-            'bugs.bug_id',
-            'bugs.bug_severity',
-            'bugs.priority',
-            'bugs.rep_platform',
-            'bugs.assigned_to',
-            'bugs.bug_status',
-            'bugs.resolution',
-            'bugs.short_desc',
-            'map_assigned_to.login_name',
+        my @searchfields = qw(
+            bug_id
+            bug_severity
+            priority
+            rep_platform
+            assigned_to
+            bug_status
+            resolution
+            short_desc
         );
         # A new Bugzilla::CGI object needs to be created to allow
         # Bugzilla::Search to execute a saved query.  It's exceedingly weird,
